@@ -5,9 +5,9 @@
 ###
 ### Plot-method(s) for fitted hhh4() models
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2018 Sebastian Meyer
-### $Revision: 2070 $
-### $Date: 2018-01-22 17:50:23 +0100 (Mon, 22. Jan 2018) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2019 Sebastian Meyer
+### $Revision: 2351 $
+### $Date: 2019-02-20 09:25:44 +0100 (Wed, 20. Feb 2019) $
 ################################################################################
 
 
@@ -346,7 +346,9 @@ plotHHH4_maps <- function (x,
 ### Map of estimated random intercepts of a specific component
 ###
 
-plotHHH4_ri <- function (x, component, labels = FALSE, sp.layout = NULL,
+plotHHH4_ri <- function (x, component, exp = FALSE,
+                         at = list(n = 10), col.regions = cm.colors(100),
+                         colorkey = TRUE, labels = FALSE, sp.layout = NULL,
                          gpar.missing = list(col="darkgrey", lty=2, lwd=2),
                          ...)
 {
@@ -360,6 +362,24 @@ plotHHH4_ri <- function (x, component, labels = FALSE, sp.layout = NULL,
     map <- as(x$stsObj@map, "SpatialPolygonsDataFrame")
     if (length(map) == 0L) stop("'x$stsObj' has no map")
     map$ranef <- ranefmatrix[,comp][row.names(map)]
+    .range <- c(-1, 1) * max(abs(map$ranef), na.rm = TRUE)  # 0-centered
+    if (exp) {
+        map$ranef <- exp(map$ranef)
+        .range <- exp(.range)
+    }
+
+    if (is.list(at)) {
+        at <- modifyList(list(n = 10, range = .range), at)
+        at <- if (exp) {
+            stopifnot(at$range[1] > 0)
+            scales::log_breaks(n = at$n)(at$range)
+        } else {
+            seq(at$range[1L], at$range[2L], length.out = at$n)
+        }
+        if (exp && isTRUE(colorkey))
+            colorkey <- list(at = log(at),
+                             labels = list(at = log(at), labels = at))
+    }
 
     if (is.list(gpar.missing) && any(is.na(map$ranef))) {
         sp.layout <- c(sp.layout,
@@ -371,7 +391,8 @@ plotHHH4_ri <- function (x, component, labels = FALSE, sp.layout = NULL,
     }
 
     spplot(map[!is.na(map$ranef),], zcol = "ranef",
-           sp.layout = sp.layout, ...)
+           sp.layout = sp.layout, col.regions = col.regions,
+           at = at, colorkey = colorkey, ...)
 }
 
 
@@ -452,8 +473,7 @@ createLambda <- function (object)
         return(Lambda)
     }
 
-    meanHHH <- meanHHH(object$coefficients, terms(object),
-                       subset=seq_len(nTime))
+    exppreds <- get_exppreds_with_offsets(object)
 
     W <- getNEweights(object)
     Wt <- if (is.null(W)) {
@@ -469,13 +489,13 @@ createLambda <- function (object)
         type <- "diagonal"
         function (t) {
             stopifnot(isScalar(t) && t > 0 && t <= nTime)
-            diag(meanHHH$ar.exppred[t,], nUnit, nUnit)
+            diag(exppreds$ar[t,], nUnit, nUnit)
         }
     } else {
         function (t) {
             stopifnot(isScalar(t) && t > 0 && t <= nTime)
-            Lambda <- meanHHH$ne.exppred[t,] * t(Wt(t))
-            diag(Lambda) <- diag(Lambda) + meanHHH$ar.exppred[t,]
+            Lambda <- exppreds$ne[t,] * t(Wt(t))
+            diag(Lambda) <- diag(Lambda) + exppreds$ar[t,]
             Lambda
         }
     }
@@ -483,6 +503,23 @@ createLambda <- function (object)
     Lambda
 }
 
+## extract exppreds multiplied with offsets
+## note: theta = coef(object) would also work since psi is not involved here
+get_exppreds_with_offsets <- function (object,
+                                       subset = seq_len(nrow(object$stsObj)),
+                                       theta = object$coefficients)
+{
+    model <- terms.hhh4(object)
+    means <- meanHHH(theta, model, subset = subset)
+    res <- sapply(X = c("ar", "ne", "end"), FUN = function (comp) {
+        exppred <- means[[paste0(comp, ".exppred")]]
+        offset <- object$control[[comp]]$offset
+        if (length(offset) > 1) offset <- offset[subset,,drop=FALSE]
+        exppred * offset
+    }, simplify = FALSE, USE.NAMES = TRUE)
+    res
+}
+
 ## determine the dominant eigenvalue of the Lambda matrix
 maxEV <- function (Lambda,
                    symmetric = isSymmetric.matrix(Lambda),
